=== modified file 'txaws/ec2/client.py'
--- txaws/ec2/client.py	2011-05-12 14:48:26 +0000
+++ txaws/ec2/client.py	2011-08-19 16:09:39 +0000
@@ -658,9 +658,20 @@
             if ip_permissions is None:
                 ip_permissions = ()
             for ip_permission in ip_permissions:
+
+                # openstack doesn't handle self authorized groups properly
+                # XXX this is an upstream problem and should be addressed there
+                # lp bug #829609
                 ip_protocol = ip_permission.findtext("ipProtocol")
-                from_port = int(ip_permission.findtext("fromPort"))
-                to_port = int(ip_permission.findtext("toPort"))
+                from_port = ip_permission.findtext("fromPort")
+                to_port = ip_permission.findtext("toPort")
+
+                if from_port:
+                    from_port = int(from_port)
+
+                if to_port:
+                    to_port = int(to_port)
+
                 for groups in ip_permission.findall("groups/item") or ():
                     user_id = groups.findtext("userId")
                     group_name = groups.findtext("groupName")

=== modified file 'txaws/ec2/tests/test_client.py'
--- txaws/ec2/tests/test_client.py	2011-05-12 14:38:37 +0000
+++ txaws/ec2/tests/test_client.py	2011-08-29 19:51:50 +0000
@@ -522,6 +522,38 @@
         d = ec2.describe_security_groups("WebServers")
         return d.addCallback(check_result)
 
+    def test_describe_security_groups_with_openstack(self):
+        """
+        L{EC2Client.describe_security_groups} can work with openstack
+        responses, which may lack proper port information for
+        self-referencing group. Verifying that the response doesn't
+        cause an internal error, workaround for nova launchpad bug
+        #829609.
+        """
+        class StubQuery(object):
+
+            def __init__(stub, action="", creds=None, endpoint=None,
+                         other_params={}):
+                self.assertEqual(action, "DescribeSecurityGroups")
+                self.assertEqual(creds.access_key, "foo")
+                self.assertEqual(creds.secret_key, "bar")
+                self.assertEqual(other_params, {"GroupName.1": "WebServers"})
+
+            def submit(self):
+                return succeed(
+                    payload.sample_describe_security_groups_with_openstack)
+
+        def check_result(security_groups):
+            [security_group] = security_groups
+            self.assertEquals(security_group.name, "WebServers")
+            self.assertEqual(
+                security_group.allowed_groups[0].group_name, "WebServers")
+
+        creds = AWSCredentials("foo", "bar")
+        ec2 = client.EC2Client(creds, query_factory=StubQuery)
+        d = ec2.describe_security_groups("WebServers")
+        return d.addCallback(check_result)
+
     def test_create_security_group(self):
         """
         L{EC2Client.create_security_group} returns a C{Deferred} that

=== modified file 'txaws/s3/client.py'
--- txaws/s3/client.py	2011-04-14 19:50:30 +0000
+++ txaws/s3/client.py	2011-09-22 16:57:38 +0000
@@ -54,11 +54,17 @@
             if not self.object_name.startswith("/"):
                 path += "/"
             path += self.object_name
+        elif self.bucket is not None and not path.endswith("/"):
+            path += "/"
         return path
 
     def get_url(self):
-        return "%s://%s%s" % (
-            self.endpoint.scheme, self.get_host(), self.get_path())
+        if self.endpoint.port is not None:
+            return "%s://%s:%d%s" % (
+                self.endpoint.scheme, self.get_host(), self.endpoint.port, self.get_path())
+        else:
+            return "%s://%s%s" % (
+                self.endpoint.scheme, self.get_host(), self.get_path())
 
 
 class S3Client(BaseClient):
@@ -389,12 +395,16 @@
         """
         Get an S3 resource path.
         """
-        resource = "/"
-        if self.bucket:
-            resource += self.bucket
-            if self.object_name:
-                resource += "/%s" % self.object_name
-        return resource
+        path = "/"
+        if self.bucket is not None:
+            path += self.bucket
+        if self.bucket is not None and self.object_name:
+            if not self.object_name.startswith("/"):
+                path += "/"
+            path += self.object_name
+        elif self.bucket is not None and not path.endswith("/"):
+            path += "/"
+        return path
 
     def sign(self, headers):
         """Sign this query using its built in credentials."""

=== modified file 'txaws/s3/tests/test_client.py'
--- txaws/s3/tests/test_client.py	2011-04-14 19:50:30 +0000
+++ txaws/s3/tests/test_client.py	2011-09-22 16:57:38 +0000
@@ -34,7 +34,7 @@
 
     def test_get_path_with_bucket(self):
         url_context = client.URLContext(self.endpoint, bucket="mystuff")
-        self.assertEquals(url_context.get_path(), "/mystuff")
+        self.assertEquals(url_context.get_path(), "/mystuff/")
 
     def test_get_path_with_bucket_and_object(self):
         url_context = client.URLContext(
@@ -62,6 +62,29 @@
             url_context.get_url(),
             "http://localhost/mydocs/notes.txt")
 
+    def test_custom_port_endpoint(self):
+        test_uri='http://0.0.0.0:12345/'
+        endpoint = AWSServiceEndpoint(uri=test_uri)
+        self.assertEquals(endpoint.port, 12345)
+        self.assertEquals(endpoint.scheme, 'http')
+        context = client.URLContext(service_endpoint=endpoint,
+                bucket="foo",
+                object_name="bar")
+        self.assertEquals(context.get_host(), '0.0.0.0')
+        self.assertEquals(context.get_url(), test_uri + 'foo/bar')
+
+    def test_custom_port_endpoint_https(self):
+        test_uri='https://0.0.0.0:12345/'
+        endpoint = AWSServiceEndpoint(uri=test_uri)
+        self.assertEquals(endpoint.port, 12345)
+        self.assertEquals(endpoint.scheme, 'https')
+        context = client.URLContext(service_endpoint=endpoint,
+                bucket="foo",
+                object_name="bar")
+        self.assertEquals(context.get_host(), '0.0.0.0')
+        self.assertEquals(context.get_url(), test_uri + 'foo/bar')
+
+
 URLContextTestCase.skip = s3clientSkip
 
 
@@ -611,7 +634,7 @@
     def test_get_canonicalized_resource(self):
         query = client.Query(action="PUT", bucket="images")
         result = query.get_canonicalized_resource()
-        self.assertEquals(result, "/images")
+        self.assertEquals(result, "/images/")
 
     def test_get_canonicalized_resource_with_object_name(self):
         query = client.Query(
@@ -619,6 +642,12 @@
         result = query.get_canonicalized_resource()
         self.assertEquals(result, "/images/advicedog.jpg")
 
+    def test_get_canonicalized_resource_with_slashed_object_name(self):
+        query = client.Query(
+            action="PUT", bucket="images", object_name="/advicedog.jpg")
+        result = query.get_canonicalized_resource()
+        self.assertEquals(result, "/images/advicedog.jpg")
+
     def test_sign(self):
         query = client.Query(action="PUT", creds=self.creds)
         signed = query.sign({})

=== modified file 'txaws/testing/payload.py'
--- txaws/testing/payload.py	2011-03-26 12:40:36 +0000
+++ txaws/testing/payload.py	2011-08-29 19:51:50 +0000
@@ -178,6 +178,43 @@
 """ % (version.ec2_api,)
 
 
+sample_describe_security_groups_with_openstack = """\
+<?xml version="1.0"?>
+<DescribeSecurityGroupsResponse xmlns="http://ec2.amazonaws.com/doc/%s/">
+  <requestId>7d4e4dbd-0a33-4d3a-864a-b5ce0f1c9cbf</requestId>
+  <securityGroupInfo>
+    <item>
+      <ipPermissions>
+        <item>
+          <toPort>22</toPort>
+          <ipProtocol>tcp</ipProtocol>
+          <ipRanges>
+             <item><cidrIp>0.0.0.0/0</cidrIp></item>
+          </ipRanges>
+          <groups/>
+          <fromPort>22</fromPort>
+        </item>
+        <item>
+         <toPort/>
+         <ipProtocol/>
+         <ipRanges/>
+         <groups>
+            <item>
+              <groupName>WebServers</groupName>
+              <userId>UYY3TLBUXIEON5NQVUUX6OMPWBZIQNFM</userId>
+            </item>
+         </groups>
+         <fromPort/>
+        </item>
+      </ipPermissions>
+      <groupName>WebServers</groupName>
+      <groupDescription>Web servers</groupDescription>
+      <ownerId>UYY3TLBUXIEON5NQVUUX6OMPWBZIQNFM</ownerId>
+    </item>
+  </securityGroupInfo>
+</DescribeSecurityGroupsResponse>
+""" % (version.ec2_api,)
+
 sample_describe_security_groups_result = """\
 <?xml version="1.0"?>
 <DescribeSecurityGroupsResponse xmlns="http://ec2.amazonaws.com/doc/%s/">

